#include "journalfile.hpp"

namespace gjournal
{
	JournalFile::JournalFile( JournalWindow *window )
	{
		srand( time( 0 ) );

		this->window = window;
	}

	bool JournalFile::_createNewFile( std::string file, std::string &error )
	{
		sqlitem::SQliteManager db;

		try {
			db.connect( file );
		}
		catch( sqlitem::SQliteConnectionException ex )
		{
			error = ex.what();
			return false;
		}

		std::string queries[2];
		queries[0] = "CREATE TABLE settings( First_Name VARCHAR(40), Last_Name VARCHAR(40) );";
		queries[1] = "CREATE TABLE entries( Date DATE, Entry MEDIUMTEXT, PRIMARY KEY( Date ) );";

		try {
			for( int i = 0; i <= 1; i++ )
			{
				db.query( queries[i] );
			}
		}
		catch( sqlitem::SQliteQueryException ex )
		{
			error = ex.what();
			return false;
		}

		db.disconnect();
		#ifndef WIN32
			chmod( file.c_str(), 0600 );
		#endif


		return true;
	}

	bool JournalFile::isJournalFile( std::string file )
	{
		char temp[1024] = { '\0' };
		unsigned char checksig[25] = { '\0' };

		std::ifstream in( file.c_str(), std::ios::binary );
		if( !in.is_open() ) return false;

		// get the various required bytes from the file for decryption
		in.getline( temp, 1024 );
		in.getline( temp, 5 );
		in.read( (char *)checksig, 25 );
		int size = in.gcount();
		if( !size ) return false;
		in.close();

		// check to make sure the magic numbers were present
		if( strcmp( (char *)aolib::BFCryptFile::encsig, (char *)checksig ) != 0 )
		{
			return false;
		}

		return true;
	}

	void JournalFile::_loadFileHandler( const char *f )
	{
		std::string oldTempFile = window->tempFile;
		std::string file( f );

		if( !JournalFile::isJournalFile( file ) )
		{
			Gtk::MessageDialog dialog( "The file you specified is not a Gtk Journal file, or it has been corrupted.", Gtk::MESSAGE_WARNING );
			dialog.run();
			dialog.hide();
			if( window->loading )
			{
				window->loading = false;
				window->_newFileHandler();
			}

			return;
		}

		aolib::LoginDialog login( "Please enter the username and password" );
		login.set_icon( Gdk::Pixbuf::create_from_xpm_data( gtk_journal_icon_xpm ) );
		bool check = false;

		if( !Glib::file_test( file, Glib::FILE_TEST_EXISTS ) )
		{
			Gtk::MessageDialog dialog( "Sorry, the file you specified does not exist.", Gtk::MESSAGE_WARNING );
			dialog.run();
			dialog.hide();
			if( window->loading )
			{
				window->loading = false;
				window->_newFileHandler();
			}

			return;
		}

		if( window->sql.isConnected() ) window->sql.disconnect();

		int result;
		window->tempFile = _getNewTempFile();

		while( !check )
		{
			result = login.run();
			if( result != Gtk::RESPONSE_OK )
			{
				if( window->loading )
				{
					window->_shredTempFile( oldTempFile );
					exit( -1 );
				}

				check = true;
				window->tempFile = oldTempFile;
			}
			else {
				Gtk::MessageDialog pd( "Decrypting Journal, please wait...", Gtk::MESSAGE_INFO, Gtk::BUTTONS_NONE );
				pd.set_modal();
				pd.show();

				while( Gtk::Main::events_pending() )
				{
					Gtk::Main::iteration();
				}


				try {
					aolib::BFCryptFile::decryptFile( login.getUsername(), login.getPassword(), file, window->tempFile );
					check = true;
				}
				catch( aolib::BFCryptException ex )
				{
					Gtk::MessageDialog dialog( ex.what(), Gtk::MESSAGE_WARNING );
					dialog.run();
					dialog.hide();
					check = false;
				}
				pd.hide();
			}
		}

		if( result != Gtk::RESPONSE_OK )
		{
			window->tempFile = oldTempFile;
			window->_dbConnect();
			return;
		}

		if( Globals::plainTextPassword ) delete Globals::plainTextPassword;
		if( Globals::username ) delete Globals::username;
		Globals::plainTextPassword = new std::string( login.getPassword() );
		Globals::username = new std::string( login.getUsername() );

		window->currentEntry = "";
		window->loading = false;
		window->journalFile = file;
		window->_shredTempFile( oldTempFile );
		window->_dbConnect();
		window->_todayHandler();
		window->_markMonthDays();
		window->savedFlag = true;
		std::string title = "Gtk+ Journal - ";
		std::string name = window->journalFile.c_str();
		unsigned int b = name.find_last_of( Globals::separator );
		if( b != std::string::npos )
		{
			name = name.substr( b + 1, name.length() );
		}

		if( window->journalFile != "" )	title += name;
		else title += "New Journal";
		window->set_title( title );
		window->statusBar.pop();
		window->statusBar.push( "Status: Unmodified" );
	}

	std::string JournalFile::_getNewTempFile()
	{
		std::stringstream tmpStream;
		std::string tempFile;

		tmpStream << ".tmp___gtkj___" << rand() << rand();
		tempFile = tmpStream.str();
		tempFile = Globals::TEMP_DIR + Globals::separator + tempFile;

		return tempFile;
	}

	void JournalFile::_preferences( bool saving )
	{
		sqlitem::SQliteManager sql = window->sql;
		if( saving && !Globals::plainTextPassword )
		{
			Gtk::MessageDialog dialog( *window, "Since this is a new Journal, you must set your name, username, and password before you can save.",
				Gtk::MESSAGE_INFO );
			dialog.run();
			dialog.hide();
		}

		sqlitem::SQliteResult result;

		try {
			// select the information from the settings table
			result = sql.query( "SELECT First_Name, Last_Name FROM settings" );
		}
		catch( sqlitem::SQliteQueryException ex )
		{
			window->_sqlError( ex.what() );
			return;
		}

		std::map<std::string, std::string> row;
		if( result.hasNext() ) row = result.next();

		// display the informationi dialog and populate it with
		// the information from the SQL query
		aolib::InformationDialog information;
		information.set_icon( Gdk::Pixbuf::create_from_xpm_data( gtk_journal_icon_xpm ) );

		information.setFirstName( row["First_Name"] );
		information.setLastName( row["Last_Name"] );
		if( Globals::username ) information.setUsername( *Globals::username );
		if( Globals::plainTextPassword ) information.setPassword( *Globals::plainTextPassword );
		information.show();
		int res = information.run();

		if( res == Gtk::RESPONSE_OK )
		{
			try {
				// delete the old settings
				sql.go( "DELETE FROM settings;" );

				// insert the new settings
				std::string query = "INSERT INTO settings VALUES( \"" + information.getFirstName()
					+ "\", \"" + information.getLastName() + "\" );";

				sql.go( query.c_str() );
			}
			catch( sqlitem::SQliteQueryException ex )
			{
				window->_sqlError( ex.what() );
			}

			// delete the old global username and password
			if( Globals::plainTextPassword ) delete Globals::plainTextPassword;
			if( Globals::username ) delete Globals::username;

			// create new globals with the new username and password
			Globals::plainTextPassword = new std::string( information.getPassword() );
			Globals::username = new std::string( information.getUsername() );

			// save the database so it matches the password
			if( saving ) window->_saveHandler();
			else {
				window->_journalEntryChangeHandler();
			}
		}
	}
}
